local ADDONNAME, ns = ...
ns = ns or {}

-- load locale
local L = LibStub("AceLocale-3.0"):GetLocale(ADDONNAME)

-- load libs
local LSM = LibStub("LibSharedMedia-3.0")
local LSK = LibStub("LibSink-2.0")

-- WoW APIs
local _G = _G

-- Lua API
local rawget, rawset, type, tonumber = rawget, rawset, type, tonumber

-- GLOBALS: MVAnnouncerDefaults


ns.defaultSoundName = "Gruntling Horn"
LSM:Register(LSM.MediaType.SOUND, ns.defaultSoundName, "Sound/Events/gruntling_horn_bb.ogg")

ns.objectIconTextureSize = 256
ns.OBJECTICONS_texture = "Interface\\MINIMAP\\OBJECTICONS.blp"
ns.OBJECTICONS_chatTexture = ("|T%s:%%d:%%d:0:0:%d:%d:%%d:%%d:%%d:%%d|t"):format(ns.OBJECTICONS_texture,ns.objectIconTextureSize,ns.objectIconTextureSize)
ns.OBJECTICONS_coords = setmetatable({}, {__index = function(t,k)
    local v = rawget(t, k)
    if type(v) ~= "table" then
        v = {_G.GetObjectIconTextureCoords(k)}
        rawset(t,k,v)
    end
    return v
end
})

ns.objectIconTypes = {
    [40] = "IconChest",
    [41] = "IconBossRar",
    [42] = "IconCombat",
    [43] = "IconChestEpic",
    [44] = "IconBossRarEpic",
    [45] = "IconCombatEpic",
}

ns.foundCooldowns = setmetatable({}, {__index=function(t,key)
    local v = rawget(t, key) or _G.GetTime()
    
    rawset(t, key, _G.GetTime()+ns.db.profile.cooldownTime)

    return v
end})

ns.SPLITKEY = "|"
ns.sinkTable = setmetatable({}, {__index=function(t,key)
    local v = rawget(t, key)

    if type(v) ~= "table" then
        v = LSK:Embed({})
        if key:find(ns.SPLITKEY) < 2 then
            v:SetSinkStorage({})
        else
            local objectIcon, name, sinkKey = ns.SPLITKEY:split(key)
            v:SetSinkStorage(ns.db.profile.objectIconTypeOfInterest[objectIcon][name].sinks[sinkKey])
        end
        rawset(t, key, v)
    end

    return v
end})

function ns.HexToRGBA(hex)
    local rhex, ghex, bhex, ahex = hex:sub(1,2), hex:sub(3,4), hex:sub(5,6), hex:sub(7,8)
        
    if ahex:len() == 2 then
        return tonumber(ghex, 16)/255, tonumber(bhex, 16)/255, tonumber(ahex, 16)/255, tonumber(rhex, 16)/255
    elseif bhex:len() == 2 then
        return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
    end        

    return 1.0, 1.0, 1.0, 1.0
end

function ns.RGBAToHex(r,g,b,a)
    r = r <= 1 and r >= 0 and r or 0
    g = g <= 1 and g >= 0 and g or 0
    b = b <= 1 and b >= 0 and b or 0
    a = (type(a)=="number") and (a <= 1 and a >= 0 and a) or nil

    if a == nil then    
        return ("%02x%02x%02x"):format(r*255, g*255, b*255)
    else
        return ("%02x%02x%02x%02x"):format(a*255, r*255, g*255, b*255)
    end
end

ns.defaults = {
    profile = {
        cooldownTime = 10,
        active = true,
        objectIconTypeOfInterest = {
            ['**'] = {
                active = true,
                ['**'] = MVAnnouncerDefaults,
            },
        },
    }
}